module gpio_wb_slave (
	wb_clk_i,
	wb_rst_i,
	wb_stb_i,
	wb_cyc_i,
	wb_we_i,
	wb_cti_i,
	wb_bte_i,
	wb_sel_i,
	wb_ack_o,
	wb_err_o,
	wb_rty_o,
	wb_adr_i,
	wb_dat_i,
	wb_dat_o,
	gpio_input_port,
	gpio_output_port
);
	input wire wb_clk_i;
	input wire wb_rst_i;
	input wire wb_stb_i;
	input wire wb_cyc_i;
	input wire wb_we_i;
	input wire [2:0] wb_cti_i;
	input wire [1:0] wb_bte_i;
	input wire [3:0] wb_sel_i;
	output reg wb_ack_o;
	output reg wb_err_o;
	output reg wb_rty_o;
	input wire [31:0] wb_adr_i;
	input wire [31:0] wb_dat_i;
	output reg [31:0] wb_dat_o;
	input wire [0:2] gpio_input_port;
	output reg [0:4] gpio_output_port;
	reg [31:0] ioregs_regs;
	reg [2:0] gpio_in_sync;
	reg [31:0] ioregs_regs_next;
	reg wb_ack_o_next;
	reg wb_err_o_next;
	reg wb_rty_o_next;
	reg [31:0] wb_dat_o_next;
	task proc_ioregs_transition_func;
		reg [31:0] wb_dat_i_var;
		reg [31:0] wb_dat_o_var;
		reg [31:0] regs_var;
		reg isAddressed;
		reg [2:0] gpio_temp;
		begin
			gpio_temp = 0;
			ioregs_regs_next = ioregs_regs;
			wb_ack_o_next = wb_ack_o;
			wb_dat_o_next = wb_dat_o;
			wb_err_o_next = wb_err_o;
			wb_rty_o_next = wb_rty_o;
			isAddressed = |('h62000000 & wb_adr_i) && (wb_adr_i <= 1644167172);
			if (isAddressed) begin
				wb_dat_i_var = wb_dat_i;
				regs_var = ioregs_regs;
				wb_ack_o_next = 0;
				wb_err_o_next = 0;
				wb_rty_o_next = 0;
				wb_dat_o_var = 0;
				if ((wb_stb_i == 1) && (wb_cyc_i == 1)) begin
					if (wb_we_i)
						regs_var = wb_dat_i_var;
					else
						wb_dat_o_var = regs_var;
					wb_ack_o_next = 1;
				end
				gpio_temp = gpio_in_sync;
				regs_var[7:5] = gpio_temp[2:0];
				ioregs_regs_next = regs_var;
				wb_dat_o_next = wb_dat_o_var;
			end
		end
	endtask
	always @(gpio_wb_slave.gpio_in_sync or gpio_wb_slave.wb_we_i or gpio_wb_slave.wb_cyc_i or gpio_wb_slave.wb_stb_i or gpio_wb_slave.ioregs_regs or gpio_wb_slave.wb_dat_i or gpio_wb_slave.wb_adr_i or gpio_wb_slave.wb_adr_i or gpio_wb_slave.wb_rty_o or gpio_wb_slave.wb_err_o or gpio_wb_slave.wb_dat_o or gpio_wb_slave.wb_ack_o or gpio_wb_slave.ioregs_regs) begin : proc_ioregs_transition_comb
		proc_ioregs_transition_func;
	end
	always @(posedge wb_clk_i) begin : proc_ioregs_transition_ff
		if (wb_rst_i) begin : sv2v_autoblock_1
			reg [31:0] wb_adr_i_var;
			reg [31:0] wb_dat_i_var;
			reg [31:0] wb_dat_o_var;
			reg [3:0] wb_sel_i_var;
			reg [31:0] regs_var;
			wb_adr_i_var = 0;
			wb_dat_i_var = 0;
			wb_dat_o_var = 0;
			wb_sel_i_var = 0;
			regs_var = 0;
			ioregs_regs <= 0;
		end
		else begin
			ioregs_regs <= ioregs_regs_next;
			wb_ack_o <= wb_ack_o_next;
			wb_err_o <= wb_err_o_next;
			wb_rty_o <= wb_rty_o_next;
			wb_dat_o <= wb_dat_o_next;
		end
	end
	reg [0:4] gpio_output_port_next;
	task proc_clk_gpio_out_func;
		reg [31:0] gpio_temp;
		begin
			gpio_output_port_next = gpio_output_port;
			gpio_temp = 0;
			gpio_temp = ioregs_regs;
			begin : sv2v_autoblock_2
				reg [63:0] i;
				for (i = 0; i < 5; i = i + 1)
					gpio_output_port_next[i] = gpio_temp[i];
			end
		end
	endtask
	always @(gpio_wb_slave.ioregs_regs or gpio_wb_slave.gpio_output_port) begin : proc_clk_gpio_out_comb
		proc_clk_gpio_out_func;
	end
	always @(posedge wb_clk_i) begin : proc_clk_gpio_out_ff
		if (wb_rst_i)
			;
		else
			gpio_output_port <= gpio_output_port_next;
	end
	reg [2:0] gpio_in_sync_next;
	task proc_clk_gpio_in_func;
		reg [2:0] gpio_temp;
		begin
			gpio_in_sync_next = gpio_in_sync;
			gpio_temp = 0;
			begin : sv2v_autoblock_3
				reg [63:0] i;
				for (i = 0; i < 3; i = i + 1)
					gpio_temp[i] = gpio_input_port[i];
			end
			gpio_in_sync_next = gpio_temp;
		end
	endtask
	always @(gpio_wb_slave.gpio_input_port or gpio_wb_slave.gpio_in_sync) begin : proc_clk_gpio_in_comb
		proc_clk_gpio_in_func;
	end
	always @(posedge wb_clk_i) begin : proc_clk_gpio_in_ff
		if (wb_rst_i)
			gpio_in_sync <= 0;
		else
			gpio_in_sync <= gpio_in_sync_next;
	end
endmodule
