--------------------------------------------------------------------------------
-- This file is part of the ParaNut project.
--
-- Copyright (C) 2023 Elias Schuler <elias.schuler@hs-augsburg.de>
--     Hochschule Augsburg, University of Applied Sciences
--
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
--
-- 1. Redistributions of source code must retain the above copyright notice, this
--    list of conditions and the following disclaimer.
--
-- 2. Redistributions in binary form must reproduce the above copyright notice,
--    this list of conditions and the following disclaimer in the documentation and/or
--    other materials provided with the distribution.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-- WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-- DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-- LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
-- ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-- Description:
--  GPIO Module
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library paranut;
use paranut.paranut_config.all;
use paranut.memu_lib.all;
use paranut.types.all;
use paranut.gpio.all;

entity gpio_wrapper is
  port (
    clk    : in std_logic;
    reset  : in std_logic;
    -- to/from Wishbone
    bifwbi : in busif_wishbone_out_type;
    bifwbo : out busif_wishbone_in_type;
    -- External Signals
    gpio_input_port  : in std_logic_vector(CFG_GPIO_INAMOUNT-1 downto 0);
    gpio_output_port : out std_logic_vector(CFG_GPIO_OUTAMOUNT-1 downto 0)
  );
end gpio_wrapper;

architecture rtl of gpio_wrapper is
  component gpio_wb_slave is
  port (
    wb_clk_i : IN STD_LOGIC;
    wb_rst_i : IN STD_LOGIC;
    wb_cyc_i : IN STD_LOGIC;
    wb_stb_i : IN STD_LOGIC;
    wb_we_i : IN STD_LOGIC;
    wb_cti_i : IN STD_LOGIC_VECTOR(2 downto 0);
    wb_bte_i : IN STD_LOGIC_VECTOR(1 downto 0);
    wb_sel_i : IN STD_LOGIC_VECTOR(3 downto 0);
    wb_adr_i : IN STD_LOGIC_VECTOR (31 downto 0);
    wb_dat_i : IN STD_LOGIC_VECTOR (CFG_MEMU_BUSIF_WIDTH-1 downto 0);
    wb_dat_o : OUT STD_LOGIC_VECTOR (CFG_MEMU_BUSIF_WIDTH-1 downto 0);
    wb_ack_o : OUT STD_LOGIC;
    wb_err_o : OUT STD_LOGIC;
    wb_rty_o : OUT STD_LOGIC;
    gpio_input_port : IN STD_LOGIC_VECTOR(CFG_GPIO_INAMOUNT-1 downto 0);
    gpio_output_port : OUT STD_LOGIC_VECTOR(CFG_GPIO_OUTAMOUNT-1 downto 0)
  );
  end component;


begin

    ICSCGPIO : component gpio_wb_slave
      port map (
        wb_clk_i => clk,
        wb_rst_i => reset,
        -- WISHBONE input
        wb_stb_i => bifwbi.stb_o,
        wb_cyc_i  => bifwbi.cyc_o,
        wb_we_i => bifwbi.we_o,
        wb_adr_i => bifwbi.adr_o,
        wb_dat_i  => bifwbi.dat_o,
        wb_cti_i => bifwbi.cti_o,
        wb_bte_i => bifwbi.bte_o,
        wb_sel_i => bifwbi.sel_o,
        -- WISHBONE output
        wb_ack_o => bifwbo.ack_i,
        wb_dat_o => bifwbo.dat_i,
        wb_err_o => bifwbo.err_i,
        wb_rty_o => bifwbo.rty_i,
        -- to all other modules
        gpio_input_port => gpio_input_port,
        gpio_output_port => gpio_output_port
      );


end rtl;
