extern crate bindgen;

use std::env;
use std::ffi::OsString;
use std::path::PathBuf;

fn main() {
    println!("cargo:rustc-cfg=riscv");
    println!("cargo:rustc-cfg=riscv32");

    let key = "RISCV_HOME";
    let riscv_home;
    match env::var_os(key) {
        Some(val) => riscv_home = val.into_string().unwrap(),
        None => {
            riscv_home = OsString::from("{key} is not defiend in the enviroment.")
                .into_string()
                .unwrap()
        }
    }

    let bindings = bindgen::Builder::default()
        .ctypes_prefix("core::ffi")
        .use_core()
        .clang_arg(String::from("--sysroot=") + riscv_home.as_str() + "/riscv64-unknown-elf/")
        .clang_arg(String::from("--prefix=") + riscv_home.as_str())
        .clang_arg("--target=riscv32-unknown-none-elf")
        .header("wrapper.h")
        .generate()
        .expect("Unable to generate bindings");

    let out_path = PathBuf::from(env::var("OUT_DIR").unwrap());
    bindings
        .write_to_file(out_path.join("bindings.rs"))
        .expect("Couldn't write bindings!");
}
